@@ -8,7 +8,7 @@ package IO::Async::SSL;
 use strict;
 use warnings;
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 $VERSION = eval $VERSION;
 
 use Carp;
@@ -16,6 +16,7 @@ use Carp;
 use IO::Socket::SSL 1.968 qw( $SSL_ERROR SSL_WANT_READ SSL_WANT_WRITE ); # default_ca
 use POSIX qw( EAGAIN EWOULDBLOCK );
 
+use Future 0.33; # ->catch_with_f
 use IO::Async::Handle 0.29;
 use IO::Async::Loop '0.61'; # new Listen API
 
@@ -139,7 +140,9 @@ The following extra methods are added to L<IO::Async::Loop>.
 
 =cut
 
-=head2 $loop->SSL_upgrade( %params ) ==> $stream | $socket
+=head2 SSL_upgrade
+
+   ( $stream or $socket ) = $loop->SSL_upgrade( %params )->get;
 
 This method upgrades a given stream filehandle into an SSL-wrapped stream,
 returning a future which will yield the given stream object or socket.
@@ -225,14 +228,17 @@ sub IO::Async::Loop::SSL_upgrade
 
    my %ssl_params = map { $_ => delete $params{$_} } grep m/^SSL_/, keys %params;
 
-   $socket = IO::Socket::SSL->start_SSL( $socket, _SSL_args
+   eval { $socket = IO::Socket::SSL->start_SSL( $socket, _SSL_args
       SSL_startHandshake => 0,
 
       # Required to make IO::Socket::SSL not ->close before we have a chance to remove it from the loop
       SSL_error_trap => sub { },
 
       %ssl_params,
-   ) or return $f->fail( IO::Socket::SSL::errstr(), "ssl" );
+   ) } or do {
+      chomp( my $e = $@ );
+      return $f->fail( $e, "ssl" );
+   };
 
    my $ready_method = $ssl_params{SSL_server} ? "accept_SSL" : "connect_SSL";
 
@@ -283,7 +289,9 @@ sub IO::Async::Loop::SSL_upgrade
    $f->on_ready( sub { undef $f } ); # intentional cycle
 }
 
-=head2 $loop->SSL_connect( %params ) ==> $stream
+=head2 SSL_connect
+
+   $stream = $loop->SSL_connect( %params )->get;
 
 This method performs a non-blocking connection to a given address or set of
 addresses, upgrades the socket to SSL, then yields a C<IO::Async::Stream>
@@ -298,7 +306,12 @@ This method can also upgrade an existing C<IO::Async::Stream> or subclass
 instance given as the C<handle> argument, by setting the C<reader> and
 C<writer> functions.
 
-=head2 $loop->SSL_connect( %params )
+=head2 SSL_connect (void)
+
+   $loop->SSL_connect( %params,
+      on_connected => sub { ... },
+      on_stream => sub { ... },
+   );
 
 When not returning a future, this method also supports the C<on_connected> and
 C<on_stream> continuations.
@@ -385,7 +398,9 @@ sub IO::Async::Loop::SSL_connect
    $f->on_ready( sub { undef $f } ); # intentional cycle
 }
 
-=head2 $loop->SSL_listen( %params )
+=head2 SSL_listen
+
+   $loop->SSL_listen( %params )->get;
 
 This method sets up a listening socket using the addresses given, and will
 invoke the callback each time a new connection is accepted on the socket and
@@ -451,18 +466,18 @@ sub IO::Async::Loop::SSL_listen
             $loop->SSL_upgrade(
                _SSL_args( SSL_server => 1, %ssl_params ),
                handle   => ( $stream || $socket ),
-            )->else( sub {
-               my ( $failure ) = @_;
-               $on_ssl_error->( $failure ) if $on_ssl_error;
-               return Future->new->fail( $failure, ssl => );
+            )->catch_with_f( ssl => sub {
+               my ( $f, $failure ) = @_;
+               if( $on_ssl_error ) {
+                  $on_ssl_error->( $failure );
+                  return Future->done;  # eat it
+               }
+               return $f;
             });
          });
       };
 
       $listener->configure( acceptor => $ssl_acceptor );
-   })->on_fail( sub {
-      my ( $message, $phase, @rest ) = @_;
-      $on_ssl_error->( $message, @rest ) if $phase eq "ssl";
    });
 
    return $f if defined wantarray;
@@ -478,7 +493,9 @@ The following extra methods are added to L<IO::Async::Protocol::Stream>.
 
 =cut
 
-=head2 $protocol->SSL_upgrade( %params )
+=head2 SSL_upgrade
+
+   $protocol->SSL_upgrade( %params )->get;
 
 A shortcut to calling C<< $loop->SSL_upgrade >>. This method will unconfigure
 the C<transport> of the Protocol, upgrade its underlying filehandle to SSL,
